from bmcs_cross_section.api import \
    ReinfLayer, BarLayer, EC2, MKappa, TShape
import bmcs_utils.api as bu
import matplotlib.pyplot as plt
from os.path import join
import pandas as pd
import numpy as np

bu.set_latex_mpl_format()
plot_colors = bu.plot_colors

def plot(mc, ver_data, test_name, title=None, ax=None, color_i=0):
    """
    Plot Monte Carlo and experimental data on a matplotlib axis.

    Parameters:
        mc : object
            An object with a method 'plot_mk(ax, dashed=True, color=...)' to plot moment-curvature data.
        ver_data : tuple or list
            A two-element container where:
                ver_data[0] is the x-axis data.
                ver_data[1] is the y-axis data.
        test_name : str
            A string used to label the experimental data in the legend.
        title : str, optional
            The title of the plot. Defaults to None.
        ax : matplotlib.axes.Axes, optional
            A matplotlib Axes object where the data will be plotted. If None, a new figure and axes are created.
        color_i : int, optional
            An index used to select a color from the 'plot_colors' array. Defaults to 0.

    Returns:
        tuple or None
            Returns a tuple (fig, ax) if a new figure is created. If an axes object was provided, returns None.
    """
    if ax is None:
        fig, ax = plt.subplots(figsize=(5, 3))
    mc.plot_mk(ax, dashed=True, color=plot_colors[color_i])
    k = ver_data[0]
    m = ver_data[1]
    ax.plot(k, m, c=plot_colors[color_i], label= 'Exp. (' + test_name + ')')
    if title is not None:
        ax.set_title(title)
    ax.legend()
    if 'fig' in locals():
        return fig, ax

def make_mc(form='rectangular', B=200, H=200, material_r1='steel', E_r1=200000, high_kappa=0.00005, f_cm=40, matrix='EC2', eps_cr=None, eps_cy=None, eps_cu=None, eps_tu=None, mu=None, E_cc=None, E_ct=None, f_ctm=None, z_r1=10, A_r1=100, f_t_r1=None, f_sy_r1=500, f_st_r1=None, eps_ud_r1=0.025, material_r2=None, z_r2=None, A_r2=None, E_r2=None, f_t_r2=None, f_sy_r2=None, f_st_r2=None, eps_ud_r2=0.025, n_kappa=200, n_m=200, slider=None):
    """
    Creates and configures a cross-section model with reinforcement layers and concrete properties
    for moment-curvature analysis.
    Parameters:
        form (str): Shape of the cross section. Default is 'rectangular'. Other forms may be defined.
        B (float): Width of the rectangular cross-section (if form is 'rectangular'). Default is 200.
        H (float): Height of the rectangular cross-section (if form is 'rectangular'). Default is 200.
        material_r1 (str): Material type for the first reinforcement layer. Typically 'steel' or 'carbon'.
                            Default is 'steel'.
        E_r1 (float): Young's modulus (or equivalent modulus) for the first reinforcement layer. Default is 200000.
        high_kappa (float): Upper limit for the curvature (kappa) parameter. Default is 0.00005.
        f_cm (float): Mean compressive strength of the concrete if using a simple compression model. Default is 40.
        matrix (str): Type of the concrete compression model. Supported values include 'EC2' and 'piecewise linear'.
                      Default is 'EC2'.
        eps_cr (float or None): Strain in the concrete for tension at cracking. Optional.
        eps_cy (float or None): Equivalent concrete strain for the piecewise linear compression model. Optional.
        eps_cu (float or None): Ultimate concrete strain in compression for the piecewise linear model. Optional.
        eps_tu (float or None): Ultimate strain in tension for the concrete. Optional.
        mu (float or None): Parameter related to the tension behavior of the concrete. Optional.
        E_cc (float or None): Tangent modulus in compression for the concrete. Optional.
        E_ct (float or None): Tangent modulus in tension for the concrete. Optional.
        f_ctm (float or None): Mean tensile strength of the concrete (not actively used in function). Optional.
        z_r1 (float): Position (distance from the reference axis) of the first reinforcement layer. Default is 10.
        A_r1 (float): Cross-sectional area of the first reinforcement layer. Default is 100.
        f_t_r1 (float or None): Tensile strength of the reinforcement material for the first layer (if applicable). Optional.
        f_sy_r1 (float): Yield strength of the reinforcement material for the first layer if using steel.
                         Default is 500.
        f_st_r1 (float or None): Ultimate strength of the reinforcement material for the first layer if using steel.
                                 If not provided, calculated as 1.05 * f_sy_r1.
        eps_ud_r1 (float): Strain at ultimate ductility for the first reinforcement layer. Default is 0.025.
        material_r2 (str or None): Material type for the second reinforcement layer. If None, no second layer is added.
        z_r2 (float or None): Position of the second reinforcement layer. Optional.
        A_r2 (float or None): Cross-sectional area of the second reinforcement layer. Optional.
        E_r2 (float or None): Young's modulus (or equivalent modulus) for the second reinforcement layer. Optional.
        f_t_r2 (float or None): Tensile strength of the reinforcement material for the second layer (if applicable). Optional.
        f_sy_r2 (float or None): Yield strength for the second reinforcement layer if using steel. Optional.
        f_st_r2 (float or None): Ultimate strength for the second reinforcement layer if using steel.
                                 If not provided, calculated as 1.05 * f_sy_r2.
        eps_ud_r2 (float): Strain at ultimate ductility for the second reinforcement layer. Default is 0.025.
        n_kappa (int): Number of discretization points for curvature. Default is 200.
        n_m (int): Number of discretization points for the bending moment. Default is 200.
        slider (any or None): Optional parameter to directly set the kappa_slider attribute in the model.
    Returns:
        mc (MKappa): A configured moment-curvature (MKappa) model object incorporating the concrete
                     properties, cross-sectional geometry, and reinforcement layers.
    Notes:
        - If matrix is 'piecewise linear', additional concrete traits (eps_cy and eps_cu) are applied.
        - The function applies specific trait settings depending on whether reinforcement material is 
          'steel' or 'carbon'.
        - The parameter f_ctm is included in the signature but is not explicitly used within the function.
    """
    
    mc = MKappa(low_kappa=0, 
                high_kappa=high_kappa,
                n_m=n_m,
                n_kappa=n_kappa)
    
    mc.cs_design.concrete.compression = matrix
    if slider is not None:
        mc.kappa_slider = slider
    
    if form=='rectangular':
        mc.cross_section_shape_.B = B
        mc.cross_section_shape_.H = H
    
    if matrix == 'piecewise linear':
        mc.cs_design.concrete.compression_.trait_set(
        eps_cy = eps_cy,
        eps_cu = eps_cu,
    )
    else:
        mc.cs_design.concrete.compression_.f_cm=f_cm
        
    if eps_tu is not None:
        mc.cs_design.concrete.tension_.eps_tu = eps_tu
    if eps_cr is not None:
        mc.cs_design.concrete.tension_.eps_cr = eps_cr
    if mu is not None:
        mc.cs_design.concrete.tension_.mu = mu
    if E_cc is not None:
        mc.cs_design.concrete.compression_.E_cc = E_cc
    if E_ct is not None:
        mc.cs_design.concrete.tension_.E_ct = E_ct
    
    bl1 = ReinfLayer(name='layer1', z=z_r1, A=A_r1, matmod=material_r1)
    if material_r1 == 'carbon':
        bl1.matmod_.trait_set(E=E_r1,f_t=f_t_r1)
    if material_r1 == 'steel':
        bl1.matmod_.trait_set(E_s=E_r1, f_sy=f_sy_r1, eps_ud = eps_ud_r1, f_st=1.05*f_sy_r1 if f_st_r1 is None else f_st_r1)
    mc.cross_section_layout.add_layer(bl1)
    
    if material_r2 is not None:
        bl2 = ReinfLayer(name='layer2', z=z_r2, A=A_r2, matmod=material_r2)
        if material_r2 == 'carbon':
            bl2.matmod_.trait_set(E=E_r2,f_t=f_t_r2)
        if material_r2 == 'steel':
            bl2.matmod_.trait_set(E_s=E_r2, f_sy=f_sy_r2, eps_ud = eps_ud_r2, f_st=1.05*f_sy_r2 if f_st_r2 is None else f_st_r2)
        mc.cross_section_layout.add_layer(bl2)
    
    return mc

def get_data(element1, xlsx_file, k, m, element2=False, k_faktor=1):
    """
    Retrieve and process data from an Excel file.
    This function constructs the full path to an Excel file based on the provided directory
    arguments, reads the file using pandas with the 'openpyxl' engine, applies an optional scaling
    factor to a specified column, and returns a list containing the processed data for two columns.
    Parameters:
        element1 (str): The base directory path where the Excel file is located.
        xlsx_file (str): The name of the Excel file.
        k (str): The column name to be processed with an optional scaling factor.
        m (str): The column name whose data will be retrieved without modification.
        element2 (str, optional): An additional subdirectory within element1. Defaults to False.
        k_faktor (int or float, optional): A multiplicative factor applied to the data in column 'k'. Defaults to 1.
    Returns:
        list: A list containing two pandas Series objects:
            - The first Series represents the column 'k' after applying the scaling factor.
            - The second Series represents the column 'm'.
    """
    if element2 == False:
        excel_file = join(element1, xlsx_file)
    else: 
        excel_file = join(element1, element2, xlsx_file)
    data = pd.read_excel(excel_file, engine='openpyxl')
    k = data[k]*k_faktor
    m = data[m]
    ver_data = [k, m]
    
    return ver_data